@@ -51,6 +51,9 @@
 /* Do you want support for IMAP GSSAPI authentication? (--with-gss) */
 #undef USE_GSS
 
+/* Do you want support for SSL? (--enable-ssl) */
+#undef USE_SSL
+
 /*
  * Is mail spooled to the user's home directory?  If defined, MAILPATH should
  * be set to the filename of the spool mailbox relative the the home
@@ -153,3 +156,11 @@
 /* Define to `int' if <signal.h> doesn't define.  */
 #undef sig_atomic_t
 
+/* define when your system has sys/time.h */
+#undef HAVE_SYS_TIME_H
+
+/* define when your system has sys/resource.h */
+#undef HAVE_SYS_RESOURCE_H
+
+/* define when your system has the setrlimit function */
+#undef HAVE_SETRLIMIT
