@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -83,14 +83,25 @@ static ADDRESS *mutt_expand_aliases_r (ADDRESS *a, LIST **expn)
       else
       {
 	struct passwd *pw = getpwnam (a->mailbox);
-	char buffer[256], *p;
 
 	if (pw)
 	{
-	  strfcpy (buffer, pw->pw_gecos, sizeof (buffer));
-	  if ((p = strchr (buffer, ',')))
-	    *p = 0;
-	  a->personal = safe_strdup (buffer);
+           regmatch_t pat_match[1];
+
+           /* Use regular expression to parse Gecos field.  This result of the
+            * parsing will be used as the personal ID string when the alias is
+            * expaned.
+            */
+	  if (regexec (GecosMask.rx, pw->pw_gecos, 1, pat_match, 0) == 0)
+	  {
+	    /* Malloc enough for the matching pattern + terminating NULL */
+	    a->personal = safe_malloc ((pat_match[0].rm_eo - 
+					pat_match[0].rm_so) + 1);
+	    
+	    strfcpy (a->personal, pw->pw_gecos + pat_match[0].rm_so, 
+		     pat_match[0].rm_eo - pat_match[0].rm_so + 1);
+	  }
+
 #ifdef EXACT_ADDRESS
 	  FREE (&a->val);
 #endif
@@ -128,7 +139,8 @@ ADDRESS *mutt_expand_aliases (ADDRESS *a)
   return (t);
 }
 
-/* if someone has an address like
+/* 
+ * if someone has an address like
  *	From: Michael `/bin/rm -f ~` Elkins <me@cs.hmc.edu>
  * and the user creates an alias for this, Mutt could wind up executing
  * the backtics because it writes aliases like
@@ -140,12 +152,20 @@ ADDRESS *mutt_expand_aliases (ADDRESS *a)
  * since that would get aliased as
  *	alias me Michael \\`/bin/rm -f ~\\` Elkins <me@cs.hmc.edu>
  * which still gets evaluated because the double backslash is not a quote.
+ * 
+ * Additionally, we need to quote ' and " characters - otherwise, mutt will
+ * interpret them on the wrong parsing step.
+ * 
+ * $ wants to be quoted since it may indicate the start of an environment
+ * variable.
  */
+
 static void write_safe_address (FILE *fp, char *s)
 {
   while (*s)
   {
-    if (*s == '\\' || *s == '`')
+    if (*s == '\\' || *s == '`' || *s == '\'' || *s == '"'
+	|| *s == '$')
       fputc ('\\', fp);
     fputc (*s, fp);
     s++;
