@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -1006,6 +1006,21 @@ int mutt_index_menu (void)
 	}
 	break;
 
+      case OP_EDIT_TYPE:
+
+	CHECK_MSGCOUNT;
+	CHECK_ATTACH;
+	mutt_edit_content_type (CURHDR, CURHDR->content);
+	/* if we were in the pager, redisplay the message */
+	if (menu->menu == MENU_PAGER)
+	{
+	  op = OP_DISPLAY_MESSAGE;
+	  continue;
+	}
+        else
+	  menu->redraw = REDRAW_CURRENT;
+	break;
+
       case OP_MAIN_NEXT_UNDELETED:
 
 	CHECK_MSGCOUNT;
