@@ -1,7 +1,6 @@
-/* $Id: dotlock.h,v 2.0 1998/12/12 08:06:06 roessler Rel $ */
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
- * Copyright (C) 1998 Thomas Roessler <roessler@guug.de>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1998-2000Thomas Roessler <roessler@guug.de>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -36,9 +35,12 @@
 #define DL_FL_USEPRIV	(1 << 2)
 #define DL_FL_FORCE	(1 << 3)
 #define DL_FL_RETRY	(1 << 4)
+#define DL_FL_UNLINK	(1 << 5)
+
+#define DL_FL_ACTIONS (DL_FL_TRY|DL_FL_UNLOCK|DL_FL_UNLINK)
 
 #ifndef DL_STANDALONE
-int dotlock_invoke(const char *, int, int);
+int dotlock_invoke (const char *, int, int);
 #endif
 
 #endif
