@@ -61,36 +61,16 @@ int Index_64[128] = {
     41,42,43,44, 45,46,47,48, 49,50,51,-1, -1,-1,-1,-1
 };
 
-static void state_maybe_utf8_putc(STATE *s, char c, int is_utf8, CHARSET *chs, CHARSET_MAP *map)
-{
-  if(is_utf8)
-    state_fput_utf8(s, c, chs);
-  else
-    state_prefix_putc(mutt_display_char ((unsigned char) c, map), s);
-}
-
-void mutt_decode_xbit (STATE *s, BODY *b, int istext)
+void mutt_decode_xbit (STATE *s, BODY *b, int istext, DECODER *dec)
 {
   long len = b->length;
   int c, ch;
-  
+  int l = 0;
+
   if (istext)
   {
-    CHARSET_MAP *map = NULL;
-    CHARSET *chs = NULL;
-    char *charset = mutt_get_parameter("charset", b->parameter);
-    int is_utf8 = 0;
-
-    if (s->flags & M_CHARCONV)
-    {
-      if((is_utf8 = (mutt_is_utf8(charset) && !mutt_is_utf8(Charset))))
-	chs = mutt_get_charset(Charset);
-      else
-	map = mutt_get_translation(charset, Charset);
-    }
-
     state_set_prefix(s);
-    
+
     while ((c = fgetc(s->fpin)) != EOF && len--)
     {
       if(c == '\r' && len)
@@ -103,15 +83,19 @@ void mutt_decode_xbit (STATE *s, BODY *b, int istext)
 	else 
 	  ungetc(ch, s->fpin);
       }
-	
-      state_maybe_utf8_putc(s, c, is_utf8, chs, map);
-    }
-    
-    state_reset_prefix(s);
 
-    if(is_utf8)
-      state_fput_utf8(s, '\0', chs);
-    
+      mutt_decoder_push_one (dec, c);
+      if (l++ == 1024)
+      {
+	mutt_decoder_pop_to_state (dec, s);
+	l = 0;
+      }
+    }
+
+    mutt_decoder_push (dec, NULL, 0, NULL);
+    mutt_decoder_pop_to_state (dec, s);
+
+    state_reset_prefix (s);
   }
   else
     mutt_copy_bytes (s->fpin, s->fpout, len);
@@ -129,25 +113,14 @@ static int handler_state_fgetc(STATE *s)
   return ch;
 }
 
-void mutt_decode_quoted (STATE *s, BODY *b, int istext)
+void mutt_decode_quoted (STATE *s, BODY *b, int istext, DECODER *dec)
 {
   long len = b->length;
   int ch;
-  char *charset = mutt_get_parameter("charset", b->parameter);
-  int is_utf8 = 0;
-  CHARSET *chs = NULL;
-  CHARSET_MAP *map = NULL;
-  
-  if(istext && (s->flags & M_CHARCONV))
-  {
-    if((is_utf8 = (mutt_is_utf8(charset) && !mutt_is_utf8(Charset))))
-      chs = mutt_get_charset(Charset);
-    else
-      map = mutt_get_translation(charset, Charset);
-  }
-  
+  int l = 0;
+
   state_set_prefix(s);
-  
+
   while (len > 0)
   {
     if ((ch = handler_state_fgetc(s)) == EOF)
@@ -200,34 +173,31 @@ void mutt_decode_quoted (STATE *s, BODY *b, int istext)
     {
       continue;
     }
+
     if(ch != EOF)
-      state_maybe_utf8_putc(s, ch, is_utf8, chs, map);
+    {
+      mutt_decoder_push_one (dec, ch);
+      if (l++ == 1024)
+      {
+	mutt_decoder_pop_to_state (dec, s);
+	l = 0;
+      }
+    }
   }
+
+  mutt_decoder_push (dec, NULL, 0, NULL);
+  mutt_decoder_pop_to_state (dec, s);
   
   state_reset_prefix(s);
-
-  if(is_utf8)
-    state_fput_utf8(s, '\0', chs);
 }
 
-void mutt_decode_base64 (STATE *s, BODY *b, int istext)
+void mutt_decode_base64 (STATE *s, BODY *b, int istext, DECODER *dec)
 {
   long len = b->length;
   char buf[5];
   int c1, c2, c3, c4, ch, cr = 0, i;
-  char *charset = mutt_get_parameter("charset", b->parameter);
-  CHARSET_MAP *map = NULL;
-  CHARSET *chs = NULL;
-  int is_utf8 = 0;
+  size_t l = 0;
 
-  if(istext && (s->flags & M_CHARCONV))
-  {
-    if((is_utf8 = (mutt_is_utf8(charset) && !mutt_is_utf8(Charset))))
-      chs = mutt_get_charset(Charset);
-    else
-      map = mutt_get_translation(charset, Charset);
-  }
-  
   buf[4] = 0;
 
   if (istext) state_set_prefix(s);
@@ -242,20 +212,28 @@ void mutt_decode_base64 (STATE *s, BODY *b, int istext)
 	buf[i++] = ch;
     }
     if (i != 4)
-      return; /* didn't get a multiple of four chars! */
+    {
+      dprint (2, (debugfile, "%s:%d [mutt_decode_base64()]: "
+		  "didn't get a multiple of 4 chars.\n", __FILE__, __LINE__));
+      break;
+    }
 
     c1 = base64val (buf[0]);
     c2 = base64val (buf[1]);
     ch = (c1 << 2) | (c2 >> 4);
 
     if (cr && ch != '\n') 
-      state_maybe_utf8_putc(s, '\r', is_utf8, chs, map);
+    {
+      mutt_decoder_push_one (dec, '\r');
+    }
     cr = 0;
       
     if (istext && ch == '\r')
       cr = 1;
     else
-      state_maybe_utf8_putc(s, ch, is_utf8, chs, map);
+    {
+      mutt_decoder_push_one (dec, ch);
+    }
 
     if (buf[2] == '=')
       break;
@@ -263,28 +241,46 @@ void mutt_decode_base64 (STATE *s, BODY *b, int istext)
     ch = ((c2 & 0xf) << 4) | (c3 >> 2);
 
     if (cr && ch != '\n')
-      state_maybe_utf8_putc(s, ch, is_utf8, chs, map);
+    {
+      mutt_decoder_push_one (dec, ch);
+    }
 
     cr = 0;
 
     if (istext && ch == '\r')
       cr = 1;
     else
-      state_maybe_utf8_putc(s, ch, is_utf8, chs, map);
+    {
+      mutt_decoder_push_one (dec, ch);
+    }
 
     if (buf[3] == '=') break;
     c4 = base64val (buf[3]);
     ch = ((c3 & 0x3) << 6) | c4;
 
     if (cr && ch != '\n')
-      state_maybe_utf8_putc(s, ch, is_utf8, chs, map);
+    {
+      mutt_decoder_push_one (dec, ch);
+    }
     cr = 0;
 
     if (istext && ch == '\r')
       cr = 1;
     else
-      state_maybe_utf8_putc(s, ch, is_utf8, chs, map);
+    {
+      mutt_decoder_push_one (dec, ch);
+    }
+    
+    if ((l += 3) >= 1024)
+    {
+      mutt_decoder_pop_to_state (dec, s);
+      l = 0;
+    }
   }
+
+  mutt_decoder_push (dec, NULL, 0, NULL);
+  mutt_decoder_pop_to_state (dec, s);
+
   state_reset_prefix(s);
 }
 
@@ -295,24 +291,12 @@ unsigned char decode_byte (char ch)
   return ch - 32;
 }
 
-void mutt_decode_uuencoded (STATE *s, BODY *b, int istext)
+void mutt_decode_uuencoded (STATE *s, BODY *b, int istext, DECODER *dec)
 {
   char tmps[SHORT_STRING];
   char linelen, c, l, out;
   char *pt;
-  CHARSET_MAP *map = NULL;
-  CHARSET *chs = NULL;
-  char *charset = mutt_get_parameter("charset", b->parameter);
-  int is_utf8 = 0;
   long len = b->length;
-  
-  if(istext && (s->flags & M_CHARCONV))
-  {
-    if ((is_utf8 = (mutt_is_utf8(charset) && !mutt_is_utf8(Charset))))
-      chs = mutt_get_charset(Charset);
-    else
-      map = mutt_get_translation(charset, Charset);
-  }
 
   if(istext)
     state_set_prefix(s);
@@ -342,19 +326,20 @@ void mutt_decode_uuencoded (STATE *s, BODY *b, int istext)
 	out = decode_byte (*pt) << l;
 	pt++;
 	out |= (decode_byte (*pt) >> (6 - l));
-	state_maybe_utf8_putc(s, out, is_utf8, chs, map);
+	mutt_decoder_push_one (dec, out);
 	c++;
 	if (c == linelen)
 	  break;
       }
+      mutt_decoder_pop_to_state (dec, s);
       pt++;
     }
   }
+
+  mutt_decoder_push (dec, NULL, 0, NULL);
+  mutt_decoder_pop_to_state (dec, s);
   
   state_reset_prefix(s);
-  if(is_utf8)
-    state_fput_utf8(s, '\0', chs);
-  
 }
 
 /* ----------------------------------------------------------------------------
@@ -1292,22 +1277,33 @@ static void external_body_handler (BODY *b, STATE *s)
 
 void mutt_decode_attachment (BODY *b, STATE *s)
 {
+  char *charset = mutt_get_parameter ("charset", b->parameter);
+  int istext = mutt_is_text_type (b->type, b->subtype);
+  DECODER *dec;
+
+  if (istext && s->flags & M_CHARCONV)
+    dec = mutt_open_decoder (charset, Charset);
+  else
+    dec = mutt_open_decoder (NULL, NULL);
+
   fseek (s->fpin, b->offset, 0);
   switch (b->encoding)
   {
     case ENCQUOTEDPRINTABLE:
-      mutt_decode_quoted (s, b, mutt_is_text_type (b->type, b->subtype));
+      mutt_decode_quoted (s, b, istext, dec);
       break;
     case ENCBASE64:
-      mutt_decode_base64 (s, b, mutt_is_text_type (b->type, b->subtype));
+      mutt_decode_base64 (s, b, istext, dec);
       break;
     case ENCUUENCODED:
-      mutt_decode_uuencoded (s, b, mutt_is_text_type (b->type, b->subtype));
+      mutt_decode_uuencoded (s, b, istext, dec);
       break;
     default:
-      mutt_decode_xbit (s, b, mutt_is_text_type (b->type, b->subtype));
+      mutt_decode_xbit (s, b, istext, dec);
       break;
   }
+
+  mutt_free_decoder (&dec);
 }
 
 void mutt_body_handler (BODY *b, STATE *s)
