@@ -29,19 +29,27 @@
 #include <string.h>
 #include <locale.h>
 
-int mutt_is_mail_list (ADDRESS *addr)
+static int _mutt_is_mail_list (ADDRESS *addr, LIST *p)
 {
-  LIST *p;
-
   if (addr->mailbox)
   {
-    for (p = MailLists; p; p = p->next)
+    for (;p; p = p->next)
       if (mutt_strncasecmp (addr->mailbox, p->data, mutt_strlen (p->data)) == 0)
 	return 1;
   }
   return 0;
 }
 
+int mutt_is_mail_list (ADDRESS *addr)
+{
+  return _mutt_is_mail_list (addr, MailLists);
+}
+
+int mutt_is_subscribed_list (ADDRESS *addr)
+{
+  return _mutt_is_mail_list (addr, SubscribedLists);
+}
+
 /* Search for a mailing list in the list of addresses pointed to by adr.
  * If one is found, print pfx and the name of the list into buf, then
  * return 1.  Otherwise, simply return 0.
@@ -51,7 +59,7 @@ check_for_mailing_list (ADDRESS *adr, char *pfx, char *buf, int buflen)
 {
   for (; adr; adr = adr->next)
   {
-    if (mutt_is_mail_list (adr))
+    if (mutt_is_subscribed_list (adr))
     {
       if (pfx && buf && buflen)
 	snprintf (buf, buflen, "%s%s", pfx, mutt_get_name (adr));
@@ -70,7 +78,7 @@ check_for_mailing_list_addr (ADDRESS *adr, char *buf, int buflen)
 {
   for (; adr; adr = adr->next)
   {
-    if (mutt_is_mail_list (adr))
+    if (mutt_is_subscribed_list (adr))
     {
       if (buf && buflen)
 	snprintf (buf, buflen, "%s", adr->mailbox);
@@ -85,7 +93,7 @@ static int first_mailing_list (char *buf, size_t buflen, ADDRESS *a)
 {
   for (; a; a = a->next)
   {
-    if (mutt_is_mail_list (a))
+    if (mutt_is_subscribed_list (a))
     {
       mutt_save_path (buf, buflen, a);
       return 1;
@@ -401,9 +409,14 @@ hdr_format_str (char *dest,
       break;
 
     case 'F':
-      snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
-      make_from (hdr->env, buf2, sizeof (buf2), 0);
-      snprintf (dest, destlen, fmt, buf2);
+      if (!optional)
+      {
+        snprintf (fmt, sizeof (fmt), "%%%ss", prefix);
+        make_from (hdr->env, buf2, sizeof (buf2), 0);
+        snprintf (dest, destlen, fmt, buf2);
+      }
+      else if (mutt_addr_is_user (hdr->env->from))
+        optional = 0;
       break;
 
     case 'i':
