@@ -19,46 +19,6 @@
 #include "mutt.h"
 
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
 #include <sys/stat.h>
 #include <string.h>
 #include <ctype.h>
@@ -87,7 +47,7 @@ void mutt_edit_headers (const char *editor,
     return;
   }
 
-  mutt_write_rfc822_header (ofp, msg->env, NULL, 1);
+  mutt_write_rfc822_header (ofp, msg->env, NULL, 1, 0);
   fputc ('\n', ofp);	/* tie off the header. */
 
   /* now copy the body of the message. */
@@ -125,7 +85,7 @@ void mutt_edit_headers (const char *editor,
   /* Read the temp file back in */
   ifp = fopen (path, "r");
   ofp = safe_fopen (body, "w");
-  n = mutt_read_rfc822_header (ifp, NULL, 1);
+  n = mutt_read_rfc822_header (ifp, NULL, 1, 0);
   while ((i = fread (buffer, 1, sizeof (buffer), ifp)) > 0)
     fwrite (buffer, 1, i, ofp);
   fclose (ofp);
@@ -145,8 +105,10 @@ void mutt_edit_headers (const char *editor,
   msg->env->reply_to = mutt_expand_aliases (msg->env->reply_to);
   msg->env->mail_followup_to = mutt_expand_aliases (msg->env->mail_followup_to);
 
-  /* search through the user defined headers added to see if either a * fcc:
-     or attach-file: field was specified.  */
+  /* search through the user defined headers added to see if either a 
+   * fcc: or attach-file: field was specified.  
+   */
+
   cur = msg->env->userhdrs;
   while (cur)
   {
