@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -91,6 +91,9 @@ struct option_t
 # ifndef USE_POP
 #  define USE_POP
 # endif
+# ifndef USE_SSL
+#  define USE_SSL
+# endif
 #endif
 
 struct option_t MuttVars[] = {
@@ -407,6 +410,16 @@ struct option_t MuttVars[] = {
   ** Useful to avoid the tampering certain mail delivery and transport
   ** agents tend to do with messages.
   */
+  { "envelope_from", 	DT_BOOL, R_NONE, OPTENVFROM, 0 },
+  /*
+  ** .pp
+  ** When \fIset\fP, mutt will try to derive the message's \fIenvelope\fP
+  ** sender from the "From:" header.  Note that this information is passed 
+  ** to sendmail command using the "-f" command line switch, so don't set this
+  ** option if you are using that switch in $$sendmail yourself,
+  ** or if the sendmail on your machine doesn't support that command
+  ** line switch.
+  */
   { "escape",		DT_STR,	 R_NONE, UL &EscChar, UL "~" },
   /*
   ** .pp
@@ -520,6 +533,15 @@ struct option_t MuttVars[] = {
   ** can be overridden using my_hdr (including from send-hooks) and
   ** ``$reverse_name''.
   */
+  { "gecos_mask",	DT_RX,	 R_NONE, UL &GecosMask, UL "^[^,]*" },
+  /*
+  ** .pp
+  ** A regular expression used by mutt to parse the GECOS field of a password
+  ** entry when expanding the alias.  By default the regular expression is set
+  ** to "^[^,]*" which will return the string up to the first "," encountered.
+  ** If the GECOS field contains a string like "lastname, firstname" then you
+  ** should set the gecos_regexp=".*".
+  */
   { "hdr_format",	DT_SYN,  R_NONE, UL "index_format", 0 },
   /*
   */
@@ -555,8 +577,8 @@ struct option_t MuttVars[] = {
   ** .pp
   ** When set, mutt will skip the host name part of ``$hostname'' variable
   ** when adding the domain part to addresses.  This variable does not
-  ** affect the generation, and it will not lead to the cut-off of first-
-  ** level domains.
+  ** affect the generation of Message-IDs, and it will not lead to the 
+  ** cut-off of first-level domains.
   */
   { "history",		DT_NUM,	 R_NONE, UL &HistSize, 10 },
   /*
@@ -981,6 +1003,7 @@ struct option_t MuttVars[] = {
   
 
   
+
   { "pipe_split",	DT_BOOL, R_NONE, OPTPIPESPLIT, 0 },
   /*
   ** .pp
