@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1999   Thomas Roessler <roessler@guug.de>
+ * Copyright (C) 1999-2000  Thomas Roessler <roessler@guug.de>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -45,6 +45,17 @@
 # include <getopt.h>
 #endif
 
+#ifndef HAVE_STRERROR
+#ifndef STDC_HEADERS
+extern int sys_nerr;
+extern char *sys_errlist[];
+#endif
+
+#define strerror(x) ((x) > 0 && (x) < sys_nerr) ? sys_errlist[(x)] : 0
+#endif /* !HAVE_STRERROR */
+
+extern int optind;
+
 #define BUFFSIZE 2048
 
 enum output_formats_t
