@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
  *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -75,6 +75,17 @@ static void print_enriched_string (int attr, unsigned char *s, int do_color)
   }
 }
 
+static void menu_make_entry (char *s, int l, MUTTMENU *menu, int i) 
+{
+  if (menu->dialog) 
+  {
+    strncpy (s, menu->dialog[i], l);
+    menu->current = -1; /* hide menubar */
+  }
+  else
+    menu->make_entry (s, l, menu, i);
+}
+
 void menu_pad_string (char *s, size_t l)
 {
 #if !defined(HAVE_BKGDSET) && !defined (USE_SLANG_CURSES)
@@ -144,7 +155,7 @@ void menu_redraw_index (MUTTMENU *menu)
   {
     if (i < menu->max)
     {
-      menu->make_entry (buf, sizeof (buf), menu, i);
+      menu_make_entry (buf, sizeof (buf), menu, i);
       menu_pad_string (buf, sizeof (buf));
 
       if (option (OPTARROWCURSOR))
@@ -190,6 +201,12 @@ void menu_redraw_index (MUTTMENU *menu)
 void menu_redraw_motion (MUTTMENU *menu)
 {
   char buf[STRING];
+
+  if (menu->dialog) 
+  {
+    menu->redraw &= ~REDRAW_MOTION;
+    return;
+  }
   
   move (menu->oldcurrent + menu->offset - menu->top, 0);
   SETCOLOR (MT_COLOR_NORMAL);
@@ -204,7 +221,7 @@ void menu_redraw_motion (MUTTMENU *menu)
     if (menu->redraw & REDRAW_MOTION_RESYNCH)
     {
       clrtoeol ();
-      menu->make_entry (buf, sizeof (buf), menu, menu->oldcurrent);
+      menu_make_entry (buf, sizeof (buf), menu, menu->oldcurrent);
       menu_pad_string (buf, sizeof (buf));
       move (menu->oldcurrent + menu->offset - menu->top, 3);
       print_enriched_string (menu->color(menu->oldcurrent), (unsigned char *) buf, 1);
@@ -222,12 +239,12 @@ void menu_redraw_motion (MUTTMENU *menu)
     /* erase the current indicator */
     attrset (menu->color (menu->oldcurrent));
     clrtoeol ();
-    menu->make_entry (buf, sizeof (buf), menu, menu->oldcurrent);
+    menu_make_entry (buf, sizeof (buf), menu, menu->oldcurrent);
     menu_pad_string (buf, sizeof (buf));
     print_enriched_string (menu->color(menu->oldcurrent), (unsigned char *) buf, 1);
 
     /* now draw the new one to reflect the change */
-    menu->make_entry (buf, sizeof (buf), menu, menu->current);
+    menu_make_entry (buf, sizeof (buf), menu, menu->current);
     menu_pad_string (buf, sizeof (buf));
     SETCOLOR (MT_COLOR_INDICATOR);
     BKGDSET (MT_COLOR_INDICATOR);
@@ -244,7 +261,7 @@ void menu_redraw_current (MUTTMENU *menu)
   char buf[STRING];
   
   move (menu->current + menu->offset - menu->top, 0);
-  menu->make_entry (buf, sizeof (buf), menu, menu->current);
+  menu_make_entry (buf, sizeof (buf), menu, menu->current);
   menu_pad_string (buf, sizeof (buf));
 
   if (option (OPTARROWCURSOR))
@@ -272,6 +289,25 @@ void menu_redraw_current (MUTTMENU *menu)
   menu->redraw &= REDRAW_STATUS;
 }
 
+void menu_redraw_prompt (MUTTMENU *menu)
+{
+  if (menu->dialog) 
+  {
+    if (option (OPTMSGERR)) 
+    {
+      sleep (1);
+      unset_option (OPTMSGERR);
+    }
+
+    if (*Errorbuf)
+      mutt_clear_error ();
+
+    SETCOLOR (MT_COLOR_NORMAL);
+    mvaddstr (LINES - 1, 0, menu->prompt);
+    clrtoeol ();
+  }
+}
+
 void menu_check_recenter (MUTTMENU *menu)
 {
   if (menu->max <= menu->pagelen && menu->top != 0)
@@ -331,7 +367,7 @@ void menu_next_line (MUTTMENU *menu)
 {
   if (menu->max)
   {
-    if (menu->top < menu->max - 1)
+    if (menu->top + 1 < menu->max)
     {
       menu->top++;
       if (menu->current < menu->top)
@@ -369,7 +405,7 @@ void menu_next_page (MUTTMENU *menu)
 	menu->current = menu->top;
       menu->redraw = REDRAW_INDEX;
     }
-    else if (menu->current != menu->max - 1)
+    else if (menu->current != menu->max - 1 && !menu->dialog)
     {
       menu->current = menu->max - 1;
       menu->redraw = REDRAW_MOTION;
@@ -391,7 +427,7 @@ void menu_prev_page (MUTTMENU *menu)
       menu->current = menu->top + menu->pagelen - 1;
     menu->redraw = REDRAW_INDEX;
   }
-  else if (menu->current)
+  else if (menu->current && !menu->dialog)
   {
     menu->current = 0;
     menu->redraw = REDRAW_MOTION;
@@ -470,7 +506,7 @@ void menu_half_up (MUTTMENU *menu)
       menu->current = menu->top + menu->pagelen - 1;
     menu->redraw = REDRAW_INDEX;
   }
-  else if (menu->current)
+  else if (menu->current && !menu->dialog)
   {
     menu->current = 0;
     menu->redraw = REDRAW_MOTION;
@@ -490,7 +526,7 @@ void menu_half_down (MUTTMENU *menu)
 	menu->current = menu->top;
       menu->redraw = REDRAW_INDEX;
     }
-    else if (menu->current != menu->max - 1)
+    else if (menu->current != menu->max - 1 && !menu->dialog)
     {
       menu->current = menu->max - 1;
       menu->redraw = REDRAW_INDEX;
@@ -570,7 +606,7 @@ static int menu_search_generic (MUTTMENU *m, regex_t *re, int n)
 {
   char buf[LONG_STRING];
 
-  m->make_entry (buf, sizeof (buf), m, n);
+  menu_make_entry (buf, sizeof (buf), m, n);
   return (regexec (re, buf, 0, NULL, 0));
 }
 
@@ -590,7 +626,18 @@ MUTTMENU *mutt_new_menu (void)
 
 void mutt_menuDestroy (MUTTMENU **p)
 {
+  int i;
+
   safe_free ((void **) &(*p)->searchBuf);
+
+  if ((*p)->dialog) 
+  {
+    for (i=0; i < (*p)->max; i++)
+      safe_free ((void **) &(*p)->dialog[i]);
+
+    safe_free ((void **) (*p)->dialog);
+  }
+
   safe_free ((void **) p);
 }
 
@@ -653,6 +700,50 @@ static int menu_search (MUTTMENU *menu, int op)
   return (-1);
 }
 
+static int menu_dialog_translate_op (int i)
+{
+  switch (i)
+  {
+    case OP_NEXT_ENTRY:   
+      return OP_NEXT_LINE;
+    case OP_PREV_ENTRY:	  
+      return OP_PREV_LINE;
+    case OP_CURRENT_TOP:   case OP_FIRST_ENTRY:  
+      return OP_TOP_PAGE;
+    case OP_CURRENT_BOTTOM:    case OP_LAST_ENTRY:	  
+      return OP_BOTTOM_PAGE;
+    case OP_CURRENT_MIDDLE: 
+      return OP_MIDDLE_PAGE; 
+  }
+  
+  return i;
+}
+
+static int menu_dialog_dokey (MUTTMENU *menu, int *ip)
+{
+  event_t ch;
+  char *p;
+
+  ch = mutt_getch ();
+
+  if (ch.ch == -1)
+  {
+    *ip = -1;
+    return 0;
+  }
+
+  if (ch.ch && (p = strchr (menu->keys, ch.ch)))
+  {
+    *ip = OP_MAX + (p - menu->keys + 1);
+    return 0;
+  }
+  else
+  {
+    mutt_ungetch (ch.op ? 0 : ch.ch, ch.op ? ch.op : 0);
+    return -1;
+  }
+}
+
 int mutt_menuLoop (MUTTMENU *menu)
 {
   int i = OP_NULL;
@@ -669,7 +760,8 @@ int mutt_menuLoop (MUTTMENU *menu)
       return (OP_REDRAW);
     }
 
-    menu_check_recenter (menu);
+    if (!menu->dialog)
+      menu_check_recenter (menu);
 
     if (menu->redraw & REDRAW_STATUS)
       menu_redraw_status (menu);
@@ -680,13 +772,22 @@ int mutt_menuLoop (MUTTMENU *menu)
     else if (menu->redraw == REDRAW_CURRENT)
       menu_redraw_current (menu);
 
+    if (menu->dialog)
+      menu_redraw_prompt (menu);
+    
     menu->oldcurrent = menu->current;
 
+
     /* move the cursor out of the way */
     move (menu->current - menu->top + menu->offset,
 	  (option (OPTARROWCURSOR) ? 2 : COLS-1));
 
     mutt_refresh ();
+    
+    /* try to catch dialog keys before ops */
+    if (menu->dialog && menu_dialog_dokey (menu, &i) == 0)
+      return i;
+		    
     i = km_dokey (menu->menu);
     if (i == OP_TAG_PREFIX)
     {
@@ -723,7 +824,12 @@ int mutt_menuLoop (MUTTMENU *menu)
     if (i == -1)
       continue;
 
-    mutt_clear_error ();
+    if (!menu->dialog)
+      mutt_clear_error ();
+
+    /* Convert menubar movement to scrolling */
+    if (menu->dialog) 
+      i = menu_dialog_translate_op (i);
 
     switch (i)
     {
@@ -779,7 +885,7 @@ int mutt_menuLoop (MUTTMENU *menu)
       case OP_SEARCH_REVERSE:
       case OP_SEARCH_NEXT:
       case OP_SEARCH_OPPOSITE:
-	if (menu->search)
+	if (menu->search && !menu->dialog) /* Searching dialogs won't work */
 	{
 	  menu->oldcurrent = menu->current;
 	  if ((menu->current = menu_search (menu, i)) != -1)
@@ -792,7 +898,10 @@ int mutt_menuLoop (MUTTMENU *menu)
 	break;
 
       case OP_JUMP:
-	menu_jump (menu);
+	if (menu->dialog)
+	  mutt_error _("Jumping is not implemented for dialogs.");
+	else
+	  menu_jump (menu);
 	break;
 
       case OP_ENTER_COMMAND:
@@ -807,11 +916,11 @@ int mutt_menuLoop (MUTTMENU *menu)
 	break;
 
       case OP_TAG:
-	if (menu->tag)
+	if (menu->tag && !menu->dialog)
 	{
 	  if (menu->max)
 	  {
-	    short i = menu->tag (menu, menu->current);
+	    int i = menu->tag (menu, menu->current);
 	    menu->tagged += i;
 	    if (i && option (OPTRESOLVE) && menu->current < menu->max - 1)
 	    {
