@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -269,8 +269,10 @@ enum
   OPTDELETEUNTAG,
   OPTEDITHDRS,
   OPTENCODEFROM,
+  OPTENVFROM,
   OPTFASTREPLY,
   OPTFCCATTACH,
+  OPTFCCCLEAR,
   OPTFOLLOWUPTO,
   OPTFORCENAME,
   OPTFORWDECODE,
@@ -349,6 +351,10 @@ enum
   OPTSORTSUBTHREADS,	/* (pseudo) used when $sort_aux changes */
   OPTNEEDRESCORE,	/* (pseudo) set when the `score' command is used */
   OPTATTACHMSG,		/* (pseudo) used by attach-message */
+  OPTKEEPQUIET,		/* (pseudo) shut up the message and refresh
+			 * 	    functions while we are executing an
+			 * 	    external program.
+			 */
   
 
 
