@@ -1,7 +1,7 @@
-/* $Id: mx.h,v 2.0 1998/12/12 08:06:53 roessler Rel $ */
 /*
- * Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>
- * 
+ * Copyright (C) 1996-2000 Michael R. Elkins <me@cs.hmc.edu>
+ * Copyright (C) 1999-2000Thomas Roessler <roessler@guug.de>
+ *
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
  *     the Free Software Foundation; either version 2 of the License, or
@@ -22,6 +22,11 @@
  * generic mailbox api.  None of these functions should be called directly.
  */
 
+#ifndef _MX_H
+#define _MX_H
+
+#include "mailbox.h"
+
 /* supported mailbox formats */
 enum
 {
@@ -48,6 +53,7 @@ int mbox_close_mailbox (CONTEXT *);
 int mbox_lock_mailbox (CONTEXT *, int, int);
 int mbox_parse_mailbox (CONTEXT *);
 int mmdf_parse_mailbox (CONTEXT *);
+void mbox_unlock_mailbox (CONTEXT *);
 
 int mh_read_dir (CONTEXT *, const char *);
 int mh_sync_mailbox (CONTEXT *);
@@ -55,7 +61,12 @@ int mh_check_mailbox (CONTEXT *, int *);
 int mh_parse_sequences (CONTEXT *, const char *);
 
 int maildir_read_dir (CONTEXT *);
-void maildir_create_filename (const char *, HEADER *, char *, char *);
+
+int maildir_commit_message (CONTEXT *, MESSAGE *, HEADER *);
+int mh_commit_message (CONTEXT *, MESSAGE *, HEADER *);
+
+int maildir_open_new_message (MESSAGE *, CONTEXT *, HEADER *);
+int mh_open_new_message (MESSAGE *, CONTEXT *, HEADER *);
 
 int mbox_strict_cmp_headers (const HEADER *, const HEADER *);
 int mutt_reopen_mailbox (CONTEXT *, int *);
@@ -66,5 +77,9 @@ void mx_update_tables (CONTEXT *, int);
 
 FILE *mx_open_file_lock (const char *, const char *);
 
+
 int mx_lock_file (const char *, int, int, int, int);
-int mx_unlock_file (const char *path, int fd);
+int mx_unlock_file (const char *path, int fd, int dot);
+
+
+#endif
